#!/usr/bin/env python3
import argparse
import json
import logging
import os
import sys
from glob import glob

from requests.exceptions import HTTPError

from ucca.convert import to_json, to_text
from ucca.ioutil import read_files_and_dirs
from uccaapp.download_task import ServerAccessor

try:
    from simplejson.scanner import JSONDecodeError
except ImportError:
    from json.decoder import JSONDecodeError

desc = """Convert a passage file to JSON format and upload to UCCA-App as a completed task"""

# https://github.com/omriabnd/UCCA-App/blob/master/UCCAApp_REST_API_Reference.pdf
# ucca-demo.cs.huji.ac.il or ucca.staging.cs.huji.ac.il
# upload the parse as a (completed) task:
# 0. decide which project and user you want to assign it to
# 1. POST passage (easy format)
# 2. POST task x (of type tokenization)
# 3. PUT task x (submit)
# 4. POST task y (of type annotation with parent x; this is the more complicated format)
# 5. PUT task y (submit)

USER_ID_ENV_VAR = "UCCA_APP_USER_ID"


class TaskUploader(ServerAccessor):
    def __init__(self, user_id, **kwargs):
        super().__init__(**kwargs)
        self.user = dict(id=user_id or int(os.environ[USER_ID_ENV_VAR]))
        
    def upload_tasks(self, filenames, **kwargs):
        del kwargs
        try:
            for pattern in filenames:
                filenames = glob(pattern)
                if not filenames:
                    raise IOError("Not found: " + pattern)
                for passage in read_files_and_dirs(filenames):
                    task = self.upload_task(passage)
                    print("Submitted task %d" % task["id"])
                    yield task
        except HTTPError as e:
            try:
                raise ValueError(e.response.json()) from e
            except JSONDecodeError:
                raise ValueError(e.response.text) from e

    def upload_task(self, passage):
        # Create passage
        passage_in = dict(text=to_text(passage, sentences=False)[0], type="PUBLIC", source=self.source)
        logging.debug("Creating passage: " + json.dumps(passage_in))
        passage_out = self.request("post", "passages/", json=passage_in).json()
        logging.debug("Created passage: " + json.dumps(passage_out))
        # Create tokenization task
        tok_task_in = dict(type="TOKENIZATION", status="SUBMITTED", project=self.project, user=self.user,
                           passage=passage_out, manager_comment=passage.ID, parent=None, is_demo=False, is_active=True)
        logging.debug("Creating tokenization task: " + json.dumps(tok_task_in))
        tok_task_out = self.request("post", "tasks/", json=tok_task_in).json()
        logging.debug("Created tokenization task: " + json.dumps(tok_task_out))
        # Submit tokenization task
        tok_user_task_in = dict(tok_task_out)
        tok_user_task_in.update(to_json(passage, return_dict=True, tok_task=True))
        logging.debug("Submitting tokenization task: " + json.dumps(tok_user_task_in))
        tok_user_task_out = self.request("put", "user_tasks/%d/submit" % tok_task_out["id"],
                                         json=tok_user_task_in).json()
        logging.debug("Submitted tokenization task: " + json.dumps(tok_user_task_out))
        # Create annotation task
        ann_task_in = dict(tok_task_in)
        ann_task_in.update(dict(parent=tok_task_out, type="ANNOTATION"))
        logging.debug("Creating annotation task: " + json.dumps(ann_task_in))
        ann_task_out = self.request("post", "tasks/", json=ann_task_in).json()
        logging.debug("Created annotation task: " + json.dumps(ann_task_out))
        # Submit annotation task
        ann_user_task_in = dict(ann_task_out)
        ann_user_task_in.update(
            to_json(passage, return_dict=True, tok_task=tok_user_task_out, all_categories=self.layer["categories"]))
        logging.debug("Submitting annotation task: " + json.dumps(ann_user_task_in))
        ann_user_task_out = self.request("put", "user_tasks/%d/submit" % ann_task_out["id"],
                                         json=ann_user_task_in).json()
        assert not ann_user_task_in["annotation_units"] or ann_user_task_out["annotation_units"], "No annotation_units"
        logging.debug("Submitted annotation task: " + json.dumps(ann_user_task_out))
        return ann_user_task_out

    @staticmethod
    def add_arguments(argparser):
        argparser.add_argument("filenames", nargs="+", help="passage file names to convert and upload")
        argparser.add_argument("--user-id", type=int, help="user id, otherwise set by " + USER_ID_ENV_VAR)
        ServerAccessor.add_arguments(argparser)


def main(**kwargs):
    list(TaskUploader(**kwargs).upload_tasks(**kwargs))


if __name__ == "__main__":
    argument_parser = argparse.ArgumentParser(description=desc)
    TaskUploader.add_arguments(argument_parser)
    main(**vars(argument_parser.parse_args()))
    sys.exit(0)



#!/usr/bin/env python3
import argparse
import json
import logging
import os
import sys

import requests

from ucca.convert import from_json, CONVERTERS, TO_FORMAT
from ucca.ioutil import write_passage

desc = """Download task from UCCA-App and convert to a passage in standard format"""

DEFAULT_SERVER = "http://ucca-demo.cs.huji.ac.il"
API_PREFIX = "/api/v1/"
SERVER_ADDRESS_ENV_VAR = "UCCA_APP_SERVER_ADDRESS"
AUTH_TOKEN_ENV_VAR = "UCCA_APP_AUTH_TOKEN"
EMAIL_ENV_VAR = "UCCA_APP_EMAIL"
PASSWORD_ENV_VAR = "UCCA_APP_PASSWORD"
PROJECT_ID_ENV_VAR = "UCCA_APP_PROJECT_ID"
SOURCE_ID_ENV_VAR = "UCCA_APP_SOURCE_ID"


class ServerAccessor(object):
    def __init__(self, server_address, email, password, auth_token, project_id, source_id, verbose, **kwargs):
        if verbose:
            logging.basicConfig(level=logging.DEBUG)
        server_address = server_address or os.environ.get(SERVER_ADDRESS_ENV_VAR, DEFAULT_SERVER)
        self.prefix = server_address + API_PREFIX
        self.headers = {}  # Needed for self.request (login)
        token = auth_token or os.environ.get(AUTH_TOKEN_ENV_VAR) or self.request("post", "login", json=dict(
            email=email or os.environ[EMAIL_ENV_VAR],
            password=password or os.environ[PASSWORD_ENV_VAR])).json()["token"]
        self.headers["Authorization"] = "Token " + token
        # Get source
        source_id = source_id or int(os.environ[SOURCE_ID_ENV_VAR])
        logging.debug("Getting source %d" % source_id)
        self.source = self.request("get", "sources/%d/" % source_id).json()
        logging.debug("Got source: " + json.dumps(self.source))
        # Get project
        project_id = project_id or int(os.environ[PROJECT_ID_ENV_VAR])
        logging.debug("Getting project %d" % project_id)
        self.project = self.request("get", "projects/%d/" % project_id).json()
        logging.debug("Got project: " + json.dumps(self.project))
        # Get layer
        layer_id = self.project["layer"]["id"]
        logging.debug("Getting layer %d" % layer_id)
        self.layer = self.request("get", "layers/%d/" % layer_id).json()
        logging.debug("Got layer: " + json.dumps(self.layer))

    def request(self, method, url_suffix, **kwargs):
        response = requests.request(method, self.prefix + str(url_suffix), headers=self.headers, **kwargs)
        response.raise_for_status()
        return response

    @staticmethod
    def add_arguments(argparser):
        argparser.add_argument("--server-address", help="UCCA-App server, otherwise set by " + SERVER_ADDRESS_ENV_VAR)
        argparser.add_argument("--email", help="UCCA-App email, otherwise set by " + EMAIL_ENV_VAR)
        argparser.add_argument("--password", help="UCCA-App password, otherwise set by " + PASSWORD_ENV_VAR)
        argparser.add_argument("--auth-token", help="authorization token (required only if email or password missing), "
                                                    "otherwise set by " + AUTH_TOKEN_ENV_VAR)
        argparser.add_argument("--project-id", type=int, help="project id, otherwise set by " + PROJECT_ID_ENV_VAR)
        argparser.add_argument("--source-id", type=int, help="source id, otherwise set by " + SOURCE_ID_ENV_VAR)
        argparser.add_argument("-v", "--verbose", action="store_true", help="detailed output")


class TaskDownloader(ServerAccessor):
    def download_tasks(self, task_ids, **kwargs):
        for task_id in task_ids:
            yield self.download_task(task_id, **kwargs)

    def download_task(self, task_id, write=True, out_format=None, binary=None, out_dir=None, prefix=None, **kwargs):
        del kwargs
        logging.debug("Getting task " + str(task_id))
        task = self.request("get", "user_tasks/" + str(task_id)).json()
        logging.debug("Got task: " + json.dumps(task))
        passage = from_json(task, all_categories=self.layer["categories"])
        if write:
            write_passage(passage, out_format, binary, out_dir, prefix, TO_FORMAT.get(out_format))
        return passage

    @staticmethod
    def add_arguments(argparser):
        argparser.add_argument("task_ids", nargs="+", type=int, help="IDs of tasks to download and convert")
        TaskDownloader.add_write_arguments(argparser)
        ServerAccessor.add_arguments(argparser)

    @staticmethod
    def add_write_arguments(argparser):
        argparser.add_argument("-f", "--out-format", choices=CONVERTERS, help="output file format (default: UCCA)")
        argparser.add_argument("-o", "--out-dir", default=".", help="output directory")
        argparser.add_argument("-p", "--prefix", default="", help="output filename prefix")
        argparser.add_argument("-b", "--binary", action="store_true", help="write in binary format (.pickle)")


def main(**kwargs):
    list(TaskDownloader(**kwargs).download_tasks(**kwargs))


if __name__ == "__main__":
    argument_parser = argparse.ArgumentParser(description=desc)
    TaskDownloader.add_arguments(argument_parser)
    main(**vars(argument_parser.parse_args()))
    sys.exit(0)
